!
! Module: omflatgen_mod
!
! Authors: Jon Rainger - MSSL
!          Chris Brindle - MSSL
! Description: Generates full window flat fields
!
! Comments:
!
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!
! History : Created June 99 - JFR
!           Modified 19 Aug 99 - 
!
!           Modified (CB) 19 June 2000 - 
!              Subroutine SortFiles added by CB to
!              return an index to the ordering of 
!              each of the files in a slew observation
!              which gives the order in terms of 
!              increasing windowX0 value. The index
!              is then used so that the files areno
!              used in the correct order from the 
!              file. Subroutine Sort added by CB to 
!              do the actual sorting for one set of 
!              slew files. Check for negative pixel
!              counts added. 
!          13th July 2000 (CB) call added to create a flatfield 
!              image through a call to "CAL_omPixelSenseVariation"
!              without a need for the four images.
!          20th September - Program changed so that -
!              1) The slew files are taken from the INDEX file
!                 nearest the required time for the flatfield.
!              2) The parameter "sigtonoise" added so that the 
!                 user specifies the required signal-to-noise ratio
!                 and the co-adding of slew files will be terminated
!                 when this value has been achieved.    
!         13.12.2001 - The code to use the INDEX file has been removed
!                 now always creates template-image.
!***************************************************************
module omflatgen_mod

    
   use param
   use cal
   use dal
   use errorHandling  
   use sla
  
   implicit none
   TYPE IMAGESIZE
       integer width
       integer height
   END TYPE IMAGESIZE
   private

   public omflatgen

contains

!---------------------------------------------------------------------
subroutine omflatgen
!---------------------------------------------------------------------

   character(len=255) :: outset                                     ! Output filename.
                                                                    
   type(DataSetT) :: outDataSetH                                    ! Output dataset handle.
                                                                    
   type(ArrayT) :: outArrayH                                        ! Output array handle.
                                                                     
   real(kind=single), dimension(:, :), pointer :: outarrayP         ! Integer array pointer
   integer(kind=int32), dimension(2) :: imageDim                    ! Image dimension.
   real(kind=single), dimension(: , :), allocatable :: flatFieldMap   ! Stores the flatfield image
  
   integer(kind=int32) :: allocErr    ! Allocate status error                                                                  
   TYPE(IMAGESIZE) imageDimensions 
   
   character(len=100) :: comment    
   
!********************************************************************  

   imageDimensions%width = 1024
   imageDimensions%height = 1024 
   
   ! Allocate memory for the flatfield image array.
   allocate(flatFieldMap(0 : 1023, 0 : 1023), stat = allocErr)
   if (allocErr /= 0) then
       call fatal("allocError", "Can't allocate memory for internal array")
   end if 


   call CAL_setState(OM, 1, 0, accuracyLevel = ACCURACY_HIGH)
   call CAL_setState(filterID = UVW1)
   
    comment = "Used CAL template" 
    
    call CreateTemplateImage(flatFieldMap, imageDimensions)
    

   !*******************************************************
   ! Create the fits file containing the created flatfield.
   !*******************************************************
   outset = stringParameter("outset")
   outDataSetH = dataSet(outset, create)
   
   !****************************************************
   !  Add attributes.
   !****************************************************
   call setAttribute(outDataSetH, "TELESCOP", "XMM", "Telescope (mission) name")
   call setAttribute(outDataSetH, "INSTRUME", "OM", "Instrument name")
   call setAttribute(outDataSetH, "DATAMODE", "IMAGING", "Instrument mode")
   call setAttribute(outDataSetH, "OBS_ID","0123456789" , "Observation identifier")
   call setAttribute(outDataSetH, "CONTENT","OM FLATFIELD" , "Contents")
   call setAttribute(outDataSetH, "HDUCLASS","OGIP" , "Format conforms to OGIP/GSFC conventions")
   call setAttribute(outDataSetH, "HDUCLAS1","IMAGE" , "File contains an image")
   call setAttribute(outDataSetH, "HDUCLAS2","DETMAP" , "Detector map")
   call setAttribute(outDataSetH, "BINBPE", "T", "BPE binning enabled")
   call setAttribute(outDataSetH, "BINAX1", 0, " ", "CCD x-axis binning 2**N")
   call setAttribute(outDataSetH, "BINAX2", 0, " ", "CCD y-axis binning 2**M")
   call setAttribute(outDataSetH, "WINDOWX0", 0, " ", "Bottom left x co-ordinate of window")
   call setAttribute(outDataSetH, "WINDOWY0", 0, " ", "Bottom left y co-ordinate of window")
   call setAttribute(outDataSetH, "WINDOWDX", 2048, " ", "X dimension of window (centroided pixels")
   call setAttribute(outDataSetH, "WINDOWDY", 2048, " ", "Y dimension of window (centroided pixels")
   call setAttribute(outDataSetH, "ORIGIN", trim(comment), "How flatfield was produced")
   
   !*****************************************************
   ! Add the flatfield array to the file
   !*****************************************************
   imageDim(1) = 1024
   imageDim(2) = 1024

   outArrayH = addArray(outDataSetH, "PRIMARY", real32, imageDim)

   outArrayP => real32Array2Data(outArrayH)
   outArrayP = flatFieldMap
   deallocate(flatFieldMap)


!  Release handles on data.

   call release(outArrayH)
   call release(outDataSetH)

   end subroutine omflatgen

!**********************************************************************
! subroutine CreateTemplateImage
! Creates an artificial flatfield image if there are no 
! slew observations.
! CAL_omPixelSenseVariation obtains a flatfield image describing the
! pixel-to-pixel sensitivity variations caused by the detector. The 
! returned image is at maximum detector resolution, ie., in
! centroided pixel space (information copied from CAL document.
! The argument countRateFrameRatio
! LEDMapP will contain a two-dimensional sensitivity map; values
! denote deviations from average throughput; the returned image
! is at maximum detector resolution (centroided pixel space)
!********************************************************************
subroutine CreateTemplateImage(image, imageDimensions)
   implicit none
   real(kind = single), dimension(0 :, 0 :), intent(inout) :: image
   TYPE(IMAGESIZE), intent(in) :: imageDimensions
   real(kind = double) :: countRateFrameRatio
   real(kind = single), dimension(:, :), pointer :: sensArray

   countRateFrameRatio = 0.0d0
   call CAL_omPixelSenseVariation(0, 0, imageDimensions%width, &
              imageDimensions%height, countRateFrameRatio, sensArray)

   image = sensArray
   call CAL_releaseMemory(sensArray)
   
end subroutine CreateTemplateImage 

end module omflatgen_mod




















